import Heartbeat, ImageCalibration, WebcamUI, SoundManager;
from VideoCapture import Device;
from Tkinter import Tk, Button, Label, Scale, Frame;
from PIL import Image, ImageStat;
import time;
import random;
from pyFMOD import *;

def new():
  return GameState();

class GameState:

  def __init__(self):
    self.heartBeat = Heartbeat.Heartbeat(1);
    self.heartBeat.addListener(self);
    self.imageCalibration = ImageCalibration.ImageCalibration(self);
    #scale sound manager so that 80 pixels = 1 meter
    self.soundManager = SoundManager.SoundManager(200);
    self.soundManager.load("start", "sounds/underway_8k.wav");
    self.soundManager.play("start");
    self.soundManager.load("in flight", "sounds/whistleBall.wav"); 
    #self.soundManager.load("in flight", "sounds/bee3d.wav", False); #True);
    #self.soundManager.load("in flight", "sounds/haha.wav", False); #True);
    #self.soundManager.load("hit", "sounds/clap.mp3");
    self.soundManager.load("miss", "sounds/MissAndClapMono.wav");
    self.soundManager.load("hit", "sounds/HitAndLaughMono.wav");
    self.soundManager.load("out of bounds", "sounds/out_of_bounds.wav");
    self.ballPos = (0,0,0);
    self.activeThrow = False;
    self.randomThrowing = False;
    self.distanceScale=100;


    print "before doUI";
    self.targetWidth = 20;
    self.targetHeight = 20;
    self.heartBeat.start();
    self.doUI();
    print "after doUI";
    self.heartBeat.stop();
    

  def __del__(self):
    self.heartBeat.stop();

  def __getattribute__(self, heartBeat):
    return self.heartBeat;

  def __getattribute__(self, imageCalibration):
    return self.imageCalibration;

  def doUI(self):
    tk = Tk();

    dev = Device();
    webcamUI = WebcamUI.new(dev, self.imageCalibration, self.getAnnotatedImageWithTargets);
    webcamUI.start(2);
    size = self.imageCalibration.getSize();
    self.heartBeat.addListener(webcamUI);

    frame = Frame(tk);

    label = Label(frame, text="Frames Per Second");
    label.pack(side="left");
    self.fps = Scale(frame, from_=1.0, to=15.0, orient="horizontal", command = self.heartBeat.setFPS);
    self.fps.set(5);
    self.fps.pack(side="left");
    button = Button(frame, text="Flip Speakers", command=self.soundManager.flipSoundSides);
    button.pack(side="left");
    frame.pack();

    frame = Frame(tk);
    label=Label(frame, text="Target Width");
    label.pack(side="left");
    self.targetWidth = Scale(frame, from_=10, to=80, orient="horizontal");
    self.targetWidth.set(40);
    self.targetWidth.pack(side="left");
    label=Label(frame, text="Target Height");
    label.pack(side="left");
    self.targetHeight = Scale(frame, from_=10, to=80, orient="vertical");
    self.targetHeight.set(40);
    self.targetHeight.pack(side="left");
    frame.pack();

    frame = Frame(tk);
    label=Label(frame, text="Target X");
    label.pack(side="left");
    self.targetX= Scale(frame, from_=0, to=size[0], orient="horizontal", command=self.setBallX);
    self.targetX.set(0);
    self.targetX.pack(side="left");
    label=Label(frame, text="Target Y");
    label.pack(side="left");
    self.targetY= Scale(frame, from_=0, to=size[1], orient="vertical", command=self.setBallY);
    self.targetY.set(0);
    self.targetY.pack(side="left");
    frame.pack();
    frame = Frame(tk);

    label=Label(frame, text="Match Threshold");
    label.pack(side="left");
    thresh = Scale(frame, from_=0.5, to=4.0, resolution="0.1", orient="horizontal", command=self.imageCalibration.setThreshold);
    thresh.set(2.0);
    thresh.pack(side="left");
    label=Label(frame, text="Collision Threshold");
    label.pack(side="left");
    self.collisionThreshold = Scale(frame, from_=0.0, to=1.0, resolution="0.1", orient="horizontal");
    self.collisionThreshold.set(0.5);
    self.collisionThreshold.pack(side="left");
    label = Label(frame, text="FSOUND Distance");
    label.pack(side="left");
    distance = Scale(frame, from_=0.0001, to=200, resolution="0.001", orient="horizontal", command=self.setFsoundDistance);
    distance.set(100);
    distance.pack(side="left");
    frame.pack();

    self.labelWhere = Label(tk, text="Target Boundary: ???");
    self.labelWhere.pack();

    frame = Frame(tk);
    b = Button(frame, text="Throw Ball", command=self.throwBall);
    b.pack(side="left");
    self.btnRandom = Button(frame, text="Throw Ball Randomly", command=self.handleRandomThrow);
    self.btnRandom.pack(side="left");
    frame.pack();

    b = Button(tk, text="done", command=tk.destroy);
    b.pack();

    print "before mainloop";
    tk.mainloop();
    print "after mainloop";

  def setFsoundDistance(self, value):
    print "setting distance to: " + value;
    self.distanceScale=float(value);
    #FSOUND_3D_SetDistanceFactor(float(value));
    #FSOUND_Update();

  def targetTracked(self, box):
    #callback for image calibration
    self.labelWhere["text"] = "Target Boundary: " + str(box);
    self.soundManager.setListenerPos((box[0]+box[2])/2, (box[1]+box[3])/2);

  def throwBall(self):
    if (not self.activeThrow):
      #com = self.imageCalibration.centerOfMass;
      #self.throwBallToPoint(com);
      self.outOfBounds = False;
      self.throwBallToPoint(self.ballPos);

  def handleRandomThrow(self):
    if (not self.activeThrow):
      self.randomThrowing = not self.randomThrowing;
      if (self.randomThrowing):
	self.btnRandom["text"] = "Stop Random Throws";
      else:
	self.btnRandom["text"] = "Throw Ball Randomly";

  def throwRandomBall(self):
      size = self.imageCalibration.getSize();
      x = random.randint(0, size[0]);
      self.targetX.set(x);
      y = random.randint(0, size[1]);
      self.targetY.set(y);
      self.ballPos = (x, y);
      self.throwBall();
      

  def throwBallToPoint(self, place):
    if (not self.activeThrow):
      com = self.imageCalibration.centerOfMass;
      if (com[0] == -1):
	self.soundManager.play("out of bounds");
      else:
        self.startTime = time.time();
        self.ballPos=(place[0], place[1], self.soundManager.getTime("in flight"));
        #print "throwing ball at: " + str(self.ballPos);
        self.activeThrow = True;
        self.soundManager.play("in flight");
        self.soundManager.setPosVec("in flight", self.scaleVector(self.ballPos));
        self.soundManager.setListenerPosVec(self.scaleVector([com[0], com[1], 0]));

  def onClock(self, notifTime):
    if (self.activeThrow):
      offset = time.time() - notifTime;
      print "offset = " + str(offset);
      zPos = self.ballPos[2] - offset;  
      self.ballPos = (self.ballPos[0], self.ballPos[1], zPos);
      self.soundManager.setPosVec("in flight", self.scaleVector(self.ballPos));
      #print "ball moved to " + str(self.ballPos);
      com = self.imageCalibration.centerOfMass;
      #print "player moved to " + str(com);
      self.soundManager.setListenerPosVec(self.scaleVector([com[0], com[1], 0]));
      if (com[0] < 0) and (not self.outOfBounds):
	self.soundManager.play("out of bounds");
	self.outOfBounds = True;
      if (self.outOfBounds) and (com[0] >= 0):
	self.outOfBounds = False;
      if (self.ballPos[2] < 0):
        self.soundManager.stop("in flight");
	print "ellapsed time: " + str(time.time() - self.startTime);
        if (self.testForCollision()):
	  self.soundManager.play("hit");
	  self.soundManager.setPosVec("hit", self.scaleVector(self.ballPos));
          self.soundManager.setListenerPosVec(self.scaleVector([com[0], com[1], 0]));
        else:
	  self.soundManager.play("miss");
	  self.soundManager.setPosVec("miss", self.scaleVector(self.ballPos));
          self.soundManager.setListenerPosVec(self.scaleVector([com[0], com[1], 0]));
        self.activeThrow = False;
	#self.ballPos = (0, 0, 0);
    else:
      if (self.randomThrowing) and (random.randint(0,7) == 0):
	self.throwRandomBall();


  def testForCollision(self):
    #these should be options
    w = int(self.targetWidth.get());
    h = int(self.targetHeight.get());
    #center = self.imageCalibration.centerOfMass;
    center = self.ballPos;
    threshold = (self.imageCalibration.threshold) * (w * h * 60) * float(self.collisionThreshold.get());

    mask = Image.new("L", self.imageCalibration.segmentedImage.size, 0);
    mask.paste(1, self.boxAroundPoint(center, w, h));
    stat = ImageStat.Stat(self.imageCalibration.segmentedImage, mask);
    print "threshhold: " + str(threshold);
    print "sum: " + str(stat.sum[0]);
    return stat.sum[0] > threshold;

  def boxCenter(self, box):
    return ((box[0]+box[2])/2, (box[1] + box[3])/2);

  def boxAroundPoint(self, point, width, height):
    return (point[0]-width/2, point[1]-height/2, point[0]+width/2, point[1]+height/2);

  def crosshairsBoxen(self, point, width, height):
    return ((point[0] - int(width/2), point[1], point[0]+int(width/2), point[1]+1),
    (point[0], point[1]-int(height/2), point[0]+1, point[1]+int(height/2)));

  def getAnnotatedImageWithTargets(self, showCenterOfMass=True, showTarget=True):
    centerOfMassColor = (0, 0, 200);
    targetColor = (200, 0, 0);
    I = self.imageCalibration.annotatedImage;
    w = int(self.targetWidth.get());
    h = int(self.targetHeight.get());
    if (showCenterOfMass):
      com = self.imageCalibration.centerOfMass;
      x = self.crosshairsBoxen(com, w, h);
      I.paste(centerOfMassColor, x[0]);
      I.paste(centerOfMassColor, x[1]);
    if (showTarget):
      x = self.crosshairsBoxen(self.ballPos, w, h);
      I.paste(targetColor, x[0]);
      I.paste(targetColor, x[1]);
    return I;


  def setBallX(self, val):
    if (not self.activeThrow):
      self.ballPos = (int(val), self.ballPos[1], 0);

  def setBallY(self, val):
    if (not self.activeThrow):
      self.ballPos = (self.ballPos[0], int(val), 0);

  def scaleVector(self, vec):
    pixelsPerMeter=self.distanceScale;
    s = [0, 0, 0];
    s[0] = vec[0] * 1.0/pixelsPerMeter;
    s[1] = vec[1] * 1.0/pixelsPerMeter;
    s[2] = vec[2];
    return s;
