from pyFMOD import *;

def new():
  return SoundManager();

class SoundManager:
  
  def __init__(self, scale=1):
    self.soundMap = {};
    self.channelMap = {};
    FSOUND_Init(44100, 32, 0);
    FSOUND_3D_SetDistanceFactor(scale);
    FSOUND_Update();
    self.yUp = -1;

  def load(self, name, filename, loopMode = False):
    if self.soundMap.has_key(name):
      self.unload(name);
    if loopMode:
      mode = FSOUND_LOOP_NORMAL;
    else:
      mode = FSOUND_LOOP_OFF;
    sound = FSOUND_Sample_Load(FSOUND_FREE, filename, mode, 0);
    if (sound != 0):
      self.soundMap[name] = sound;
    return (sound != 0);

  def play(self, name):
    if self.soundMap.has_key(name):
      if self.channelMap.has_key(name):
	FSOUND_PlaySound(self.channelMap[name], self.soundMap[name]);
      else:
        channel = FSOUND_PlaySound(FSOUND_FREE, self.soundMap[name]);
	self.channelMap[name] = channel;

  def stop(self, name):
    if self.channelMap.has_key(name):
     FSOUND_StopSound(self.channelMap[name]);
  
  def setPos(self, name, x, y, z):
    if self.channelMap.has_key(name):
      FSOUND_3D_SetAttributes(self.channelMap[name], [x, y, z], [0, 0, -1]);
      #FSOUND_Update();

  def setPosVec(self, name, vec):
    self.setPos(name, vec[0], vec[1], vec[2]);
    

  def setListenerPosVec(self, v):
    # we assume that forward vector is (0, 0, 1)
    # up vecotr is (0, 1, 0)

    FSOUND_3D_Listener_SetAttributes(v, [0, 0, 0], 0, 0, self.yUp, 0, 1, 0); 
    FSOUND_Update();
     
  def setListenerPos(self, x, y, z=0):
    self.setListernerPosVec(self, [x, y, z]);

  def unload(self, name):
     #stop playing the sound then unload it
     self.stop(name);
     FSOUND_Sample_Free(self.soundMap[name]);

  def getTime(self, name):
    if (self.soundMap.has_key(name)):
      return (1.0 * FSOUND_Sample_GetLength(self.soundMap[name])) / FSOUND_Sample_GetDefaults(self.soundMap[name])[0];
    else:
      return 0;

  def flipSoundSides(self):
    self.yUp = -1 * self.yUp;
    print "Flipped sound, yUp is now: " + str(self.yUp);

  def __del__(self):
    #destructor, stop all sounds
    #then free all memory used by samples
    #then shutdown FMOD
    for name in self.channelMap.keys():
      self.unload(name);
    FSOUND_Close();

  
